@@ -16,7 +16,6 @@
 */
 
 #include <config.h>
-#include "configmake.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -27,6 +26,7 @@
 
 #include <idn2.h>
 
+#include <unitypes.h>
 #include <uniconv.h>
 #include <unistr.h>
 
@@ -86,6 +86,10 @@ Mandatory arguments to long options are mandatory for short options too.\n\
       fputs (_("\
   -l, --lookup             Lookup domain name (default)\n\
   -r, --register           Register label\n\
+"), stdout);
+      fputs (_("\
+  -T, --tr46t              Enable TR46 transitional processing\n\
+  -N, --tr46nt             Enable TR46 non-transitional processing\n\
 "), stdout);
       fputs (_("\
       --debug              Print debugging information\n\
@@ -123,12 +127,56 @@ hexdump (const char *prefix, const char *str)
 	       prefix, (unsigned long) i, u32[i]);
 }
 
+static struct gengetopt_args_info args_info;
+
+static void
+process_input (char *readbuf, int flags)
+{
+  size_t len = strlen (readbuf);
+  char *output;
+  int rc;
+
+  if (len && readbuf[len - 1] == '\n')
+    readbuf[len - 1] = '\0';
+
+  if (strcmp (readbuf, "show w") == 0)
+    {
+      puts (WARRANTY);
+      return;
+    }
+  else if (strcmp (readbuf, "show c") == 0)
+    {
+      puts (CONDITIONS);
+      return;
+    }
+
+  if (args_info.debug_given)
+    hexdump ("input", readbuf);
+
+  if (args_info.register_given)
+    rc = idn2_register_ul (readbuf, NULL, &output, flags);
+  else
+    rc = idn2_lookup_ul (readbuf, &output, flags);
+
+  if (rc == IDN2_OK)
+    {
+      if (args_info.debug_given)
+	hexdump ("output", readbuf);
+
+      printf ("%s\n", output);
+      free (output);
+    }
+  else
+    error (EXIT_FAILURE, 0, "%s: %s",
+	   args_info.register_given ? "register" : "lookup",
+	   idn2_strerror (rc));
+}
+
 int
 main (int argc, char *argv[])
 {
-  struct gengetopt_args_info args_info;
-  char readbuf[BUFSIZ];
-  unsigned cmdn = 0;
+  unsigned cmdn;
+  int flags = 0;
 
   setlocale (LC_ALL, "");
   set_program_name (argv[0]);
@@ -160,60 +208,29 @@ main (int argc, char *argv[])
     fprintf (stderr, "%s", _("Type each input string on a line by itself, "
 			     "terminated by a newline character.\n"));
 
-  do
+  if (args_info.tr46t_given)
+    flags = IDN2_TRANSITIONAL;
+  else if (args_info.tr46nt_given)
+    flags = IDN2_NONTRANSITIONAL;
+
+  for (cmdn = 0; cmdn < args_info.inputs_num; cmdn++)
+    process_input (args_info.inputs[cmdn], flags);
+
+  if (!cmdn)
     {
-      char *output;
-      int rc;
+      char *buf = NULL;
+      size_t bufsize = 0;
 
-      if (cmdn < args_info.inputs_num)
-	{
-	  strncpy (readbuf, args_info.inputs[cmdn++], BUFSIZ - 1);
-	  readbuf[BUFSIZ - 1] = '\0';
-	}
-      else if (fgets (readbuf, BUFSIZ, stdin) == NULL)
-	{
-	  if (feof (stdin))
-	    break;
+      while (getline (&buf, &bufsize, stdin) > 0)
+	process_input (buf, flags);
 
-	  error (EXIT_FAILURE, errno, "%s", _("input error"));
-	}
-
-      if (strlen(readbuf) > 0 && readbuf[strlen (readbuf) - 1] == '\n' )
-	readbuf[strlen (readbuf) - 1] = '\0';
-
-      if (strcmp (readbuf, "show w") == 0)
-	{
-	  puts (WARRANTY);
-	  continue;
-	}
-      else if (strcmp (readbuf, "show c") == 0)
-	{
-	  puts (CONDITIONS);
-	  continue;
-	}
-
-      if (args_info.debug_given)
-	hexdump ("input", readbuf);
-
-      if (args_info.register_given)
-	rc = idn2_register_ul (readbuf, NULL, &output, 0);
-      else
-	rc = idn2_lookup_ul (readbuf, &output, 0);
-
-      if (rc == IDN2_OK)
-	{
-	  if (args_info.debug_given)
-	    hexdump ("output", readbuf);
-
-	  printf ("%s\n", output);
-	}
-      else
-	error (EXIT_FAILURE, 0, "%s: %s",
-	       args_info.register_given ? "register" : "lookup",
-	       idn2_strerror (rc));
+      free (buf);
     }
-  while (!feof (stdin) && !ferror (stdin) && (args_info.inputs_num == 0 ||
-					      cmdn < args_info.inputs_num));
+
+  if (ferror (stdin))
+    error (EXIT_FAILURE, errno, "%s", _("input error"));
+
+  cmdline_parser_free (&args_info);
 
   return EXIT_SUCCESS;
 }
